/*
 * usb/endpt1.c -- USB endpoit 1 (HID) handling
 *
 * Copyright (C) 2016-2017 Tomasz Kramkowski <tk@the-tk.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <reg/usbotg.h>
#include <stddef.h>
#include <stdint.h>

#include "../uart.h"

#include "bdt.h"
#include "endpt1.h"

#define MAX_PACKET 64

static unsigned char report[4][4] = {
	{ 0x00, 5, 0, 0x00, },
	{ 0x00, 0, 5, 0x00, },
	{ 0x00, -5, 0, 0x00, },
	{ 0x00, 0, -5, 0x00, },
};
static int nextrep;

void usb_endpt1_enable(void)
{
	nextrep = 0;

	usb_bdt[1][BDT_TX][BDT_EVEN].addr = &report[0];
	usb_bdt[1][BDT_TX][BDT_EVEN].desc = USB0_BD_INIT(4, nextrep % 2);
	nextrep++;

	usb_bdt[1][BDT_TX][BDT_ODD].addr = &report[1];
	usb_bdt[1][BDT_TX][BDT_ODD].desc = USB0_BD_INIT(4, nextrep % 2);
	nextrep++;

	USB0_ENDPT(1) = BV(ENDPT_EPTXEN) | BV(ENDPT_EPHSHK);
}

void usb_endpt1_disable(void)
{
	USB0_ENDPT(1) = 0;
}

void usb_endpt1_token(uint8_t state)
{
	volatile struct usb0_bd *bd;

	bd = &usb_bdt[1][GET_BIT(state, STAT_TX)][GET_BIT(state, STAT_ODD)];

	switch (GET_BITS(bd->desc, BD_TOK_PID)) {
	case BD_TOK_PID_IN:
		bd->addr = &report[nextrep];
		bd->desc = USB0_BD_INIT(4, nextrep % 2);
		nextrep = (nextrep + 1) % 4;
		break;
	}
}
